@@ -1,21 +1,21 @@
 use std::task::{Context, Poll};
-use std::{cell::RefCell, future::Future, pin::Pin, rc::Rc, time::Duration};
+use std::{cell::RefCell, future::Future, pin::Pin, rc::Rc};
 
 use crate::codec::{AsyncRead, AsyncWrite, ReadBuf};
 use crate::framed::State;
-use crate::rt::time::{sleep, Sleep};
+use crate::time::{sleep, Sleep};
 
 #[derive(Debug)]
 enum IoWriteState {
     Processing,
-    Shutdown(Option<Pin<Box<Sleep>>>, Shutdown),
+    Shutdown(Option<Sleep>, Shutdown),
 }
 
 #[derive(Debug)]
 enum Shutdown {
     None,
     Flushed,
-    Shutdown,
+    Stopping,
 }
 
 /// Write io task
@@ -43,15 +43,8 @@ where
 
     /// Shutdown io stream
     pub fn shutdown(io: Rc<RefCell<T>>, state: State) -> Self {
-        let disconnect_timeout = state.get_disconnect_timeout() as u64;
-        let st = IoWriteState::Shutdown(
-            if disconnect_timeout != 0 {
-                Some(Box::pin(sleep(Duration::from_millis(disconnect_timeout))))
-            } else {
-                None
-            },
-            Shutdown::None,
-        );
+        let disconnect_timeout = state.get_disconnect_timeout();
+        let st = IoWriteState::Shutdown(disconnect_timeout.map(sleep), Shutdown::None);
 
         Self { st, io, state }
     }
@@ -80,15 +73,9 @@ where
                 if this.state.is_io_shutdown() {
                     log::trace!("write task is instructed to shutdown");
 
-                    let disconnect_timeout = this.state.get_disconnect_timeout() as u64;
+                    let disconnect_timeout = this.state.get_disconnect_timeout();
                     this.st = IoWriteState::Shutdown(
-                        if disconnect_timeout != 0 {
-                            Some(Box::pin(sleep(Duration::from_millis(
-                                disconnect_timeout,
-                            ))))
-                        } else {
-                            None
-                        },
+                        disconnect_timeout.map(sleep),
                         Shutdown::None,
                     );
                     return self.poll(cx);
@@ -129,7 +116,7 @@ where
                             match Pin::new(&mut *this.io.borrow_mut()).poll_shutdown(cx)
                             {
                                 Poll::Ready(Ok(_)) => {
-                                    *st = Shutdown::Shutdown;
+                                    *st = Shutdown::Stopping;
                                     continue;
                                 }
                                 Poll::Ready(Err(_)) => {
@@ -142,7 +129,7 @@ where
                                 _ => (),
                             }
                         }
-                        Shutdown::Shutdown => {
+                        Shutdown::Stopping => {
                             // read until 0 or err
                             let mut buf = [0u8; 512];
                             let mut io = this.io.borrow_mut();
@@ -164,8 +151,8 @@ where
                     }
 
                     // disconnect timeout
-                    if let Some(ref mut delay) = delay {
-                        if Pin::new(delay).poll(cx).is_pending() {
+                    if let Some(ref delay) = delay {
+                        if delay.poll_elapsed(cx).is_pending() {
                             return Poll::Pending;
                         }
                     }
