@@ -1,16 +1,14 @@
 use std::task::{Context, Poll};
-use std::{convert::TryFrom, error::Error, future::Future, net, pin::Pin, time};
+use std::{convert::TryFrom, error::Error, future::Future, net, pin::Pin};
 
-use bytes::Bytes;
-use derive_more::From;
-use futures_core::Stream;
 use serde::Serialize;
 
 use crate::http::body::{Body, BodyStream};
 use crate::http::error::HttpError;
 use crate::http::header::{self, HeaderMap, HeaderName, HeaderValue};
 use crate::http::RequestHeadType;
-use crate::rt::time::{sleep, Sleep};
+use crate::time::{sleep, Millis, Sleep};
+use crate::{util::Bytes, Stream};
 
 #[cfg(feature = "compress")]
 use crate::http::encoding::Decoder;
@@ -50,7 +48,7 @@ impl From<PrepForSendingError> for SendRequestError {
 pub enum SendClientRequest {
     Fut(
         Pin<Box<dyn Future<Output = Result<ClientResponse, SendRequestError>>>>,
-        Option<Pin<Box<Sleep>>>,
+        Option<Sleep>,
         bool,
     ),
     Err(Option<SendRequestError>),
@@ -60,10 +58,9 @@ impl SendClientRequest {
     pub(crate) fn new(
         send: Pin<Box<dyn Future<Output = Result<ClientResponse, SendRequestError>>>>,
         response_decompress: bool,
-        timeout: Option<time::Duration>,
+        timeout: Millis,
     ) -> SendClientRequest {
-        let delay = timeout.map(|d| Box::pin(sleep(d)));
-        SendClientRequest::Fut(send, delay, response_decompress)
+        SendClientRequest::Fut(send, timeout.map(sleep), response_decompress)
     }
 }
 
@@ -76,7 +73,7 @@ impl Future for SendClientRequest {
         match this {
             SendClientRequest::Fut(send, delay, _response_decompress) => {
                 if delay.is_some() {
-                    match Pin::new(delay.as_mut().unwrap()).poll(cx) {
+                    match delay.as_ref().unwrap().poll_elapsed(cx) {
                         Poll::Pending => (),
                         _ => return Poll::Ready(Err(SendRequestError::Timeout)),
                     }
@@ -132,17 +129,21 @@ impl RequestHeadType {
         self,
         addr: Option<net::SocketAddr>,
         response_decompress: bool,
-        timeout: Option<time::Duration>,
+        mut timeout: Millis,
         config: &ClientConfig,
         body: B,
     ) -> SendClientRequest
     where
         B: Into<Body>,
     {
+        if timeout.is_zero() {
+            timeout = config.timeout;
+        }
+
         SendClientRequest::new(
             config.connector.send_request(self, body.into(), addr),
             response_decompress,
-            timeout.or(config.timeout),
+            timeout,
         )
     }
 
@@ -150,7 +151,7 @@ impl RequestHeadType {
         mut self,
         addr: Option<net::SocketAddr>,
         response_decompress: bool,
-        timeout: Option<time::Duration>,
+        timeout: Millis,
         config: &ClientConfig,
         value: &T,
     ) -> SendClientRequest {
@@ -177,7 +178,7 @@ impl RequestHeadType {
         mut self,
         addr: Option<net::SocketAddr>,
         response_decompress: bool,
-        timeout: Option<time::Duration>,
+        timeout: Millis,
         config: &ClientConfig,
         value: &T,
     ) -> SendClientRequest {
@@ -207,7 +208,7 @@ impl RequestHeadType {
         self,
         addr: Option<net::SocketAddr>,
         response_decompress: bool,
-        timeout: Option<time::Duration>,
+        timeout: Millis,
         config: &ClientConfig,
         stream: S,
     ) -> SendClientRequest
@@ -228,10 +229,10 @@ impl RequestHeadType {
         self,
         addr: Option<net::SocketAddr>,
         response_decompress: bool,
-        timeout: Option<time::Duration>,
+        timeout: Millis,
         config: &ClientConfig,
     ) -> SendClientRequest {
-        self.send_body(addr, response_decompress, timeout, config, Body::Empty)
+        self.send_body(addr, response_decompress, timeout, config, Body::None)
     }
 
     fn set_header_if_none<V>(
