@@ -1,8 +1,5 @@
-use std::io::Write;
 use std::marker::PhantomData;
-use std::{cell::Cell, cmp, io, mem, ptr, ptr::copy_nonoverlapping, slice};
-
-use bytes::{BufMut, BytesMut};
+use std::{cell::Cell, cmp, io, io::Write, mem, ptr, ptr::copy_nonoverlapping, slice};
 
 use crate::http::body::BodySize;
 use crate::http::config::DateService;
@@ -11,6 +8,7 @@ use crate::http::helpers;
 use crate::http::message::{ConnectionType, RequestHeadType};
 use crate::http::response::Response;
 use crate::http::{HeaderMap, StatusCode, Version};
+use crate::util::{BufMut, BytesMut};
 
 const AVERAGE_HEADER_SIZE: usize = 30;
 
@@ -82,6 +80,7 @@ pub(super) trait MessageType: Sized {
             BodySize::Sized(len) => write_content_length(len, dst),
             BodySize::Stream => {
                 if chunked {
+                    skip_len = true;
                     dst.extend_from_slice(b"\r\ntransfer-encoding: chunked\r\n")
                 } else {
                     skip_len = false;
@@ -546,6 +545,7 @@ fn write_content_length(mut n: u64, bytes: &mut BytesMut) {
 
 unsafe fn convert_usize(mut n: u64, bytes: &mut BytesMut) {
     let mut curr: isize = 39;
+    #[allow(clippy::uninit_assumed_init)]
     let mut buf: [u8; 41] = mem::MaybeUninit::uninit().assume_init();
     buf[39] = b'\r';
     buf[40] = b'\n';
@@ -595,11 +595,10 @@ unsafe fn convert_usize(mut n: u64, bytes: &mut BytesMut) {
 mod tests {
     use std::rc::Rc;
 
-    use bytes::Bytes;
-
     use super::*;
     use crate::http::header::{HeaderValue, AUTHORIZATION};
     use crate::http::RequestHead;
+    use crate::util::Bytes;
 
     #[test]
     fn test_chunked_te() {
